#################################################################################
# Created       : Martin Foster
# Modified      : 10/12/2002
#################################################################################
#
# Post - Module part of Webchat designed to display posts based on information
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Post;						# Package name

use strict;							# Strict variable enforcement


#################################################################################
# Data Members
#################################################################################

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
        my $class = shift;
        my $self  = { };

        # Bless $self into designated class
        bless $self, $class;

        # Return self
        return $self;
}


#####################
# Get Posts
#
# Using basic information about the user will return a Statement handle (more 
# lightweight then an array) that will be used in the display the information
# gathered.

sub GetPosts
{
	#####################
	# Data members
	my $self      = shift;					# Self reference
	my $database  = shift;					# Database handle
	my $statement = shift;					# Statement handle (reference)
	my $param     = shift;					# Parameter list
	my $realm     = shift;					# Reference to realm hash

	my $data      = $database->{'HANDLE'};			# Database connection handle

	my $chat;						# Chat mode
	my $delay;						# Delay to retreive posts
	my $format;						# Formatting used	
	my $privacy;						# Privacy feature

	my $ignore;						# Query string
	my @ignore;						# Ignored users


	#####################
	# Program area

	# Determine action 
	# Default
	$delay = (time - $realm->{'SetTimeout'}) unless (defined($param->{'NUM'}));

	# Scrollback	
	if (defined($param->{'SCROLLBACK'}))
	{ 
		# Time of delay
		if (($param->{'SCROLLBACK'} !~ /^\d+$/)
		 && (length($param->{'SCROLLBACK'}) > 1))
		{
			# Set time of delay as per default
			$delay = (time - $realm->{'SetTimeout'}); 
		}

		# Position specific
		elsif ($param->{'SCROLLBACK'} =~ /^\d+$/)
		{
			# Safety
			$param->{'SCROLLBACK'} = ($param->{'SCROLLBACK'} > 500) ? 500 : $param->{'SCROLLBACK'};
			$param->{'SCROLLBACK'} = ($param->{'SCROLLBACK'} < 10)  ? 10  : $param->{'SCROLLBACK'};


			# Prepare and execute
			# Selects all post ID's for actual posts
			my $results = $data->prepare("SELECT PostIDNumber
				FROM Post
				WHERE RealmName=?
				  AND PostClass IN ('general','play')
				  AND PostTo='all'
				ORDER BY PostIDNumber DESC LIMIT ?");
 			$results->execute($param->{'ROOM'}, $param->{'SCROLLBACK'});	

		 	# Link results
			my $data = $results->fetchall_arrayref();

			# Retreive post number
			# Last result, first column
			$param->{'NUM'} = $data->[-1][0];	


			# Always define
			$param->{'NUM'} = (defined($param->{'NUM'})) ? $param->{'NUM'} : 1;


			# Close
			$results->finish();
		}


		# Cleanup
		$param->Cleanup('SCROLLBACK');
	}	


	# Format handling
	# Retreive formatting used
	($format, $chat, $privacy) = $database->DataGetFormatting($param->{'USER'});


	# Formatting
	# Full formatting
	if ($format eq 'complete')	
	{ $format = 'PostFullFormat'; }

	# Spartan formatting
	elsif ($format eq 'spartan')
	{ $format = 'PostPartialFormat'; }

	# Full less images
	elsif ($format eq 'imageless')
	{ $format = 'PostImagelessFormat'; }


	# Privacy filter
	$privacy = ($privacy eq 'yes') ? "AND PostClass <> 'private'" : '';


	# List of ignored
	($ignore, @ignore) = $self->IgnoreHandler($database, $param);

	# Number based search
	unless (defined($delay))
	{
		# Initialize and execute
		${$statement} = $data->prepare("SELECT PostIDNumber, 
					 $format, 
					 PuppeteerLogin, 
					 PostCmd, 
					 PostClass 
					FROM Post 
					WHERE PostIDNumber > ?
					  AND RealmName=?
					  AND (PostTo='all' OR PostTo=?) 
				          $privacy $ignore ORDER BY PostIDNumber");
		${$statement}->execute($param->{'NUM'}, $param->{'ROOM'}, $param->{'USER'}, @ignore);
	}

	# Time based search
	else
	{
		# Initialize and execute
		${$statement} = $data->prepare("SELECT PostIDNumber, 
					 $format, 
					 PuppeteerLogin, 
					 PostCmd,
					 PostClass 
					FROM Post 
					WHERE PostTimeStamp > ?
					  AND RealmName=?
					  AND (PostTo='all' OR PostTo=?) 
					  $privacy $ignore ORDER BY PostIDNumber");
		${$statement}->execute($delay, $param->{'ROOM'}, $param->{'USER'}, @ignore);
	}
}


#####################
# Handle Posts
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation, will also give a value to NUM once it knows the last post
# received

sub HandlePosts
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list
	my $choice   = shift;					# Choice handling

	my $statement;						# Query statement

	my $class;						# Command class
	my $cmd;						# Command handling
	my $num;						# Post number
	my $post;						# Post itself
	my $postnum;						# Post number
	my $user;						# User posting

	my %realm;						# Realm hash


	#####################
	# Program area

	# Link with hash
	$database->GetHashRealm(\%realm);


	# Retreive posts
	$self->GetPosts($database, \$statement, $param, \%realm);	

	# Run though posts and print
	while (($num, $post, $user, $cmd, $class) = $statement->fetchrow())
	{ 
		# Additional commands
		if (defined($choice))
		{
			# IC Filter
			if (($user eq $param->{'USER'}) 
			 && ($cmd eq 'play'))
			{
				# Create the 'checkbox'
				$param->{'ICPOSTS'} = 'ON' unless (defined($param->{'ICPOSTS'}));
			}

			# Regular handling
			elsif (($user eq $param->{'USER'})
			 && ($cmd eq 'general'))
			{
				# Remove the 'checkbox'
				$param->Cleanup('ICPOSTS') if (defined($param->{'ICPOSTS'}));
			}
		}


		# Skip
		next if ((defined($param->{'ICPOSTS'})) && ($class eq 'general'));


		# Post Display
		# Display seperator/post
		print $database->FetchRealmSeperator($param->{'ROOM'});

		# Display post
		print "$post\n";


		# Assign Post Number
		$postnum = $num;

		# Assignment of root post
		$param->{'ROOT'} = (defined($param->{'ROOT'})) ? $param->{'ROOT'} : $num;


	}

	# Finish query
	$statement->finish();
	
	# Change value of NUM or define
	if (defined($postnum))
	{ $param->{'NUM'} = ($postnum - 1); }
}


#####################
# Handle Stream
#
# Will display all retreived posts to screen, a fairly straightforward 
# operation.  This unlike HandlePosts will not stop until the stream
# is ended

sub HandleStream
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI Handle
	my $param    = shift;					# Parameter list

	my $statement;						# Query statement

	my $class;						# Command class
	my $cmd;						# Command handling
	my $num;						# Post number
	my $post;						# Post itself
	my $user;						# User posting

	my $postnum;						# Post number
	my $postold = 0;					# Old post number

	my $size    = 0;					# Size of posts
	my $active  = 0;					# Activity counter
	my $elapse  = 0;					# Time elapsed		

	my %realm;						# Chat hash


	#####################
	# Program area

	# Connect to hash
	$database->GetHashRealm(\%realm);


	# As long as conditions are not met
	while (($size  < $realm{'SetStreamSize'}) 
	   && ($active < $realm{'SetActive'})
	   && ($elapse < $realm{'SetElapse'}))
	{
		# Retreive posts
		$self->GetPosts($database, \$statement, $param, \%realm);	

		# Run though posts and print
		while (($num, $post, $user, $cmd, $class) = $statement->fetchrow())
		{ 
			# Additional commands
			# IC Filter
			if (($user eq $param->{'USER'}) 
			 && ($cmd eq 'play'))
			{
				# Create the 'checkbox'
				$param->{'ICPOSTS'} = 'ON' unless (defined($param->{'ICPOSTS'}));
			}

			# Regular handling
			elsif (($user eq $param->{'USER'})
			 && ($cmd eq 'general'))
			{
				# Remove the 'checkbox'
				$param->Cleanup('ICPOSTS') if (defined($param->{'ICPOSTS'}));
			}


			# Skip
			next if ((defined($param->{'ICPOSTS'})) && ($class eq 'general'));


			# Display
			# Display seperator
			print $database->FetchRealmSeperator($param->{'ROOM'});

			# Display post
			print "$post\n";
	
			# Assignment of post number
			$postnum = $num;
		
			# Assignment of root post
			$param->{'ROOT'} = (defined($param->{'ROOT'})) ? $param->{'ROOT'} : $num;


			# Possibilities of continuation
			# Determin cumulative size
			if (($user eq $param->{'USER'}) 
			 && ($cmd eq 'stream'))
			{
				# Trick and terminate
				$size  = $realm{'SetStreamSize'};

				# Increment to skip post
				$postnum++;

				# Clear loop
				last;
			}

			# Default behaviour
			else
			{
				# Simply add length
				$size += length($post);
			}

		}

		# Finish query
		$statement->finish();
	
		# Change value of NUM or define
		if (defined($postnum))
		{ 
			# Increment 
			$active = ($postold != $postnum ) ? 0 : ($active + 1); 

			# Assign PostOld and Parameter
			$postold 	= $postnum;
			$param->{'NUM'} = $postnum;
		}

		# Print comment
		print $cgi->comment("$postnum"), "\n";

		# Final size calculations
		$size += (length($postnum) + 8);
		
		# Increment Elapsed time
		$elapse++;


		# Sleep before next pass
		sleep(5);
	}

	# To retrieve last made post
	if (defined($param->{'NUM'})) { $param->{'NUM'}--; }


	# Activation button
	print $cgi->start_form();
	print $param->EmbedNormal($param->Flat(), 'MAIN', 'True'), "\n";

	# Display refresh
	$self->RefreshStream($database, $cgi, $param);

	# End of form
	print $cgi->end_form(), "\n";
}


#####################
# Ignore Handler
#
# Will create a string to append to the end of the SQL statement, as
# to ignore the specified list of people.

sub IgnoreHandler
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $database = shift;					# Database handle
	my $param    = shift;					# Parameter list

	my $string;						# Ignored string
	my @ignored;						# Ignored group

	my $count;						# Number of ignored
	my $placeholders;					# Query placeholders


	#####################
	# Program area

	# Retreive list of the damned
	$database->GetListPuppetIgnored(\@ignored, $param->{'USER'});


	# Return count
	$count = @ignored;

 	# Ignore	
	# Only if needed
	if ($count > 0)
	{
		# Generate placeholders
		$placeholders = "?," x $count;
		chop($placeholders);

		# Generate
		$string = "AND (PuppetName NOT IN ($placeholders) OR PuppetName IS NULL)";


		# Return values
		return $string, @ignored;
	}


	# No ignores
	return '', @ignored; 
}


#####################
# Refresh
#
# Quick subroutine that will create the necessary HTML used at the bottom of
# frames and stream variants.   Allows the hosts to make things look all so
# pretty without changing code.

sub Refresh
{
	#####################
	# Data members
	my $self     = shift;					# Self referencing link
	my $choice   = shift;					# Choice of the matter
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle

	my $refresh;						# Refresh panel

	my %realm;						# Realm hash


	#####################
	# Program Area

	# Link with has
	$database->GetHashRealm(\%realm);

	# Pull document
	$refresh = $database->FetchRealmRefresh($param->{'ROOM'});
	

	# Prepare
	# Scrollbacks
	my $wscrlback = $cgi->checkbox(-name=>'SCROLLBACK', -label=>$realm{'TxtScrollback'});
	my $wscrladvd = $cgi->textfield('SCROLLBACK', '', 3, 3) . " " . $realm{'TxtScrollback'};

	# Post retreival
	my $wscrlctrl = $cgi->checkbox(-name=>'GETPOSTS', -label=>$realm{'TxtRemote'});

	# Button
	my $wwrefresh =  $cgi->submit(-name=>$realm{'SetPost'}, -value=>$realm{'SetPost'}, -accesskey=>'r');


	# Search and replace
	$refresh =~ s/WSCRLBACK/$wscrlback/gsi;
	$refresh =~ s/WSCRLADVD/$wscrladvd/gsi;
	$refresh =~ s/WSCRLCTRL/$wscrlctrl/gsi;
	$refresh =~ s/WWREFRESH/$wwrefresh/gsi;


	# Display
	print "$refresh\n";
}


# Framed version
sub RefreshFramed
{ Refresh(shift, '0', @_); }

# Stream version
sub RefreshStream
{ Refresh(shift, '1', @_); }

1;								# Return true
